﻿using System;
using System.Collections.Generic;
using System.Data.OleDb;
using System.Threading.Tasks;

using DevExpress.Utils;
using DevExpress.XtraEditors;
using DevExpress.XtraGrid;
using DevExpress.XtraGrid.Views.Grid;

namespace TestProject
{
    /// <summary>
    /// 메인 폼
    /// </summary>
    public partial class MainForm : XtraForm
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 연결
        /// </summary>
        private OleDbConnection connection;

        /// <summary>
        /// 소스 리스트
        /// </summary>
        private List<OrderDetail> sourceList;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - MainForm()

        /// <summary>
        /// 생성자
        /// </summary>
        public MainForm()
        {
            InitializeComponent();

            this.connection = new OleDbConnection("Provider=Microsoft.Jet.OLEDB.4.0;Data Source=nwind.mdb");

            this.gridView.InitializeView
            (
                true,                           // Multi Select
                GridMultiSelectMode.CellSelect, // Grid Multi Select Mode
                DrawFocusRectStyle.CellFocus,   // Draw Focus Rect Style
                true,                           // Show Indicator
                true,                           // Show Column Headers
                true,                           // Allow Column Moving
                true,                           // Allow Column Resizing
                true,                           // Allow Filter
                true,                           // Allow Sort
                false,                          // Allow Cell Merge
                EditorShowMode.Default,         // Editor Show Mode
                false                           // Editable
            );

            this.gridView.OptionsBehavior.AutoPopulateColumns = true;

            this.sourceList = OrderDetail.GetOrderDetailList(this.connection); 

            this.gridControl.DataSource = this.sourceList;

            FormatConditionRuleDataUpdate formatConditionRuleDataUpdate = new FormatConditionRuleDataUpdate();

            formatConditionRuleDataUpdate.HighlightTime       = 500;
            formatConditionRuleDataUpdate.Icon.PredefinedName = "Flags3_1.png";
            formatConditionRuleDataUpdate.PredefinedName      = "Green Fill";
            formatConditionRuleDataUpdate.Trigger             = FormatConditionDataUpdateTrigger.ValueChanged;

            GridFormatRule gridFormatRule = new GridFormatRule();

            gridFormatRule.Column = this.gridView.Columns["UnitPrice"];
            gridFormatRule.Rule   = formatConditionRuleDataUpdate;

            this.gridView.FormatRules.Add(gridFormatRule);

            this.gridView.BestFitColumns();

            this.executeButton.Click += executeButton_Click;
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region 실행 버튼 클릭시 처리하기 - executeButton_Click(sender, e)

        /// <summary>
        /// 실행 버튼 클릭시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private async void executeButton_Click(object sender, EventArgs e)
        {
            this.executeButton.Enabled = false;

            await Task.Run
            (
                () =>
                {
                    Random random1 = new Random(DateTime.Now.Millisecond);
                    Random random2 = new Random(DateTime.Now.Millisecond);

                    for(int i = 0; i < 10; i++)
                    {
                        int rowHandle = random1.Next(0, 22);

                        decimal value = Convert.ToDecimal((200 * random2.NextDouble()).ToString("#,##0.00"));

                        this.gridControl.Invoke
                        (
                            new Action
                            (
                                () =>
                                {
                                    this.gridView.SetRowCellValue(rowHandle, this.gridView.Columns["UnitPrice"], value);
                                }
                            )
                        );
                    }
                }
            );

            this.executeButton.Enabled = true;
        }

        #endregion
    }
}